# This file describes the network interfaces available on your system
# and how to activate them. For more information, see interfaces(5).

# The loopback network interface
auto lo
iface lo inet loopback

# The primary network interface
auto eth0
iface eth0 inet dhcp

{% if ansible_interfaces|length > 2 %}
{% for interface in ansible_interfaces if interface != 'lo' and interface != 'eth0' %}
 
auto {{ interface }}
iface {{ interface }} inet dhcp
 
{% endfor %}
{% endif %}